﻿using UnityEngine;
using System.Collections.Generic;

public class DialogBox : MonoBehaviour {

	public Transform dialogBoxWindow;

	class Script {
		public List<string> pages;
		public bool playedOnce = false;
	}

	static Dictionary<string, Script> scripts;

	static Script currentScript;
	static int currentPage;

	// Use this for initialization
	void Start () {
		scripts = new Dictionary<string, Script>();

		Script script;

		script = new Script();
		script.pages = new List<string>();
		script.pages.Add ("Ah, hullo and welcome to The LuftWaffel, the most elite Zeppelin corps on the planet.");
		script.pages.Add ("As traders, it is our job to connect people and places, much like how the Silk Road connected the Western World to the Far East realms so long ago.");
		script.pages.Add ("Of course, as you know our once united land was torn asunder by a terrible cataclysm. Or maelstrom. Or something. Whatever, the point is that we can only transport goods via ZEPPELIN now.\n\nAlso, flying cities?");
		script.pages.Add ("As our LEAD ZEPPELIN ADMINISTRATOR, your first priority will be to bring FOOD to the cities. Without food, people will starve. And without people, there are no TAXES to buy more Zeppelins.");
		script.pages.Add ("Afterwards, you should focus on bringing LUXURY GOODS like Fruit Cakes, Fancy Hats, Vests, and Goggles to the cities so that we can raise the tax rate without complaint.");
		script.pages.Add ("Finally, if you bring STEEL and PLANKS to cities, they will expand their population. More people means more taxes, and if you reach a population of 1 MILLION, then you win!");
		script.pages.Add ("Keep in mind that most goods will need to be processed by various industry buildings before the cities will accept them.");
		script.pages.Add ("TL;DR:\n\nMiddle/Right Mouse or WASD to scroll.\n\nScroll to zoom.\n\nClick the 'NEW ZEPPELIN' button.");
		scripts.Add("Introduction", script);

		script = new Script();
		script.pages = new List<string>();
		script.pages.Add( "You've reached a population of one million!  Congratulations, now that there's so much cheap labour, we can fire you and hire someone else.\n\n(Click 'Close' to keep playing if you want.)");
		scripts.Add("Victory!", script);
		
		script = new Script();
		script.pages = new List<string>();
		script.pages.Add( "Oh no! One of your cities ran out of food and a housing ship has been lost. Remedy this quickly!" );
		scripts.Add("Lost a house!", script);
		
		script = new Script();
		script.pages = new List<string>();
		script.pages.Add( "You delivered enough STEEL and PLANKS to a city to result in a new housing complex being built. Your population is booming!" );
		scripts.Add("New Housing!", script);
		
		
		script = new Script();
		script.pages = new List<string>();
		script.pages.Add( "Good show!  Because you've delivered luxury goods to a city, we're able to raise taxes while still keeping people happy!" );
		scripts.Add("Tax Increase!", script);
		
		


		ShowScript("Introduction");
	}

	static public void ShowScript(string name, bool oneShot=false) {

		if(currentScript != null) {
			Debug.LogError("Already have a dialog open!");
			return;
		}

		Transform dialogBoxWindow = GameObject.FindObjectOfType<DialogBox>().dialogBoxWindow;

		if(scripts.ContainsKey(name) == false) {
			Debug.LogError("No script with name: " + name);
			return;
		}

		currentScript = scripts[name];

		if(currentScript.playedOnce==true && oneShot==true) {
			GameObject.FindObjectOfType<DialogBox>().OnOk();
			return;
		}

		currentScript.playedOnce = true;
		currentPage = 0;


		dialogBoxWindow.gameObject.SetActive(true);
		dialogBoxWindow.Find ("Title").GetComponent<UILabel>().text = name;
		dialogBoxWindow.Find ("Content").GetComponent<UILabel>().text = scripts[name].pages[currentPage];

		UpdateButtons();

		Time.timeScale = 0;
	}

	static public void UpdateButtons() {
		Transform dialogBoxWindow = GameObject.FindObjectOfType<DialogBox>().dialogBoxWindow;

		if(currentPage > 0) {
			dialogBoxWindow.Find("Back").gameObject.SetActive(true);
		}
		else {
			dialogBoxWindow.Find("Back").gameObject.SetActive(false);
		}

		if(currentPage < currentScript.pages.Count - 1 ) {
			dialogBoxWindow.Find("Next").gameObject.SetActive(true);
		}
		else {
			dialogBoxWindow.Find("Next").gameObject.SetActive(false);
		}
		
		if(currentPage == currentScript.pages.Count-1 ) {
			dialogBoxWindow.Find("Ok").gameObject.SetActive(true);
		}
		else {
			dialogBoxWindow.Find("Ok").gameObject.SetActive(false);
		}
	}

	public void OnNext() {
		currentPage++;
		dialogBoxWindow.Find ("Content").GetComponent<UILabel>().text = currentScript.pages[currentPage];
		UpdateButtons();
	}
	public void OnBack() {
		currentPage--;
		dialogBoxWindow.Find ("Content").GetComponent<UILabel>().text = currentScript.pages[currentPage];
		UpdateButtons();
	}
	public void OnOk() {
		dialogBoxWindow.gameObject.SetActive(false);
		currentScript = null;
		Time.timeScale = 1;
	}

	public void OnHelp() {
		ShowScript("Introduction");
	}
}
