﻿using UnityEngine;
using System.Collections.Generic;

public class ResourceStorage : MonoBehaviour {
	public static string[] ALL_GOODS = { 
		"Iron", "Coal", "Steel", "Meat", "Food", "Leather", "Wheat", "Vests", 
		"Goggles", "Fancy Hats", "Milk", "Sand", "Glass Lenses", "Brass", "Zinc",  
		"Logs", "Planks", "Fruit", "Spices", "Fruit Cakes", "Beef", "Copper"
	};
	public static string[] LUX_GOODS = {"Vests", "Goggles", "Fancy Hats", "Fruit Cakes"};

	public Dictionary<string, float> contents;
	public Dictionary<string, float> demands;

	ResourceConsumer rc;
	SkyCity skycity;

	void Start() {
		rc = GetComponent<ResourceConsumer>();
		skycity = GetComponent<SkyCity>();
	}

	public float contentTotal {
		get {
			float t = 0;
			foreach(float ct in contents.Values) {
				t += ct;
			}
			return t;
		}
	}

	public float storageLimit = 10f;	// Soft limit, mostly for production.

	public void ChangeResource(string name, float amt) {
		if(contents == null) 
			contents = new Dictionary<string, float>();

		if(contents.ContainsKey(name)==false) {
			contents[name] = amt;
		}
		else {
			contents[name] += amt;
		}

		if(contents[name] < 0) {
			Debug.LogError("Negative contents? WTF?");
		}

		//Debug.Log (gameObject.name + "'s store of '"+name+"' is: " + contents[name]);
	}

	public float GetResource(string name) {
		if(contents==null || contents.ContainsKey(name)==false) {
			return 0;
		}

		return contents[name];

	}

	public void DumpCargo(ResourceStorage target) {
		List<string> cargoNames = new List<string>( contents.Keys );
		foreach(string name in cargoNames) {
			target.ChangeResource(name, contents[name]);
			contents[name] = 0;
		}
	}

	void Update() {
		if(demands == null) {
			demands = new Dictionary<string, float>();
		}
		if(contents == null) {
			contents = new Dictionary<string, float>();
		}

		foreach(string name in ALL_GOODS) {
			if(demands.ContainsKey(name)==false)
				demands[name] = 0;
			if(contents.ContainsKey(name)==false)
				contents[name] = 0;

			if(HasInput(name)) {
				if( contents[name] >= storageLimit ) {
					demands[name] = Mathf.Clamp(demands[name] - (Time.deltaTime / 3f), -1f, 1f); 
				}
				else if( contents[name] >= storageLimit/2f) {
					// Stay constant
					demands[name] = Mathf.Clamp(demands[name] + (Time.deltaTime / 30f), -1f, 1f); 
				}
				else {
					demands[name] = Mathf.Clamp(demands[name] + (Time.deltaTime / 3f), -1f, 1f); 
				}
			}
			else if(skycity != null && (name=="Food" || name=="Steel" || name=="Planks" || name=="Vests" || name=="Goggles" || name=="Fancy Hats" || name=="Fruit Cakes")) {
				if( contents[name] >= storageLimit ) {
					demands[name] = Mathf.Clamp(demands[name] - (Time.deltaTime / 3f), -1f, 1f); 
				}
				else if( contents[name] >= storageLimit/2f) {
					demands[name] = Mathf.Clamp(demands[name] + (Time.deltaTime / 30f), -1f, 1f); 
				}
				else {
					demands[name] = Mathf.Clamp(demands[name] + (Time.deltaTime / 3f), -1f, 1f); 
				}

			}
			else if( contents[name] >= 1f ){
				// We go from 0 to -1 over 30 seconds while in an oversupply situation.
				demands[name] = Mathf.Clamp(demands[name] - (Time.deltaTime / 10f), -1f, 1f); 
			}
			else {
				demands[name] = Mathf.Lerp( demands[name], 0, Time.deltaTime / 10f );
			}
		}
	}

	bool HasInput(string name) {
		if(rc == null)
			return false;

		foreach(string i in rc.inputNames) {
			if(i==name)
				return true;
		}

		return false;
	}

	/*public void IncreaseDemandFor(string name) {
		if(demands == null) {
			demands = new Dictionary<string, float>();
		}

		if(demands.ContainsKey(name)==false)
			demands[name] = 0;

		// We go from 0 to +1 over 30 seconds while in a shortfall situation.
		demands[name] = Mathf.Clamp(demands[name] + Time.deltaTime / 30f, -1f, 1f); 
	}*/

	/// <summary>
	/// Gets the demand for.
	/// </summary>
	/// <returns>+1 is high demand, -1 is oversupplied</returns>
	/// <param name="name">Name.</param>
	public float GetDemandFor(string name) {
		if(demands==null)
			return 0;

		if(demands.ContainsKey(name)==false)
			return 0;

		return demands[name];
	}

	override public string ToString() {
		string s = "";
		foreach(string name in contents.Keys) {
			if(contents[name] >= 1f) {
				if(s != "")
					s += ", ";

				s += name + "["+contents[name].ToString("F")+"]";
			}
		}

		if(s=="")
			return "NONE";

		return s;
	}

	public string GetHighestDifferentialGood(ResourceStorage target) {
		// Limited to goods we actually possess.

		string n = "";
		float d = 0.01f;

		foreach(string name in contents.Keys) {
			if(contents[name] >= 1f && target.GetDemandFor(name) > 0 && (Differential(target, name) > d) && target.GetResource(name) <= target.storageLimit ) {
				n = name;
				d = Differential(target, n);
			}
		}

		return n;
	}

	float Differential(ResourceStorage target, string name) {
		return target.GetDemandFor(name) - GetDemandFor(name);
	}
}
