﻿using UnityEngine;
using System.Collections;

[RequireComponent(typeof(SeeksPosition))]
[RequireComponent(typeof(ResourceStorage))]
public class Route : MonoBehaviour {

	public ResourceStorage[] stops;
	int currStop=0;

	float arrivalDistance = 2f;
	float unloadTime = 1f;
	float loadTime = 1f;
	float unloadTimeLeft;
	float loadTimeLeft;
	bool unloading = true;

	SeeksPosition seeker;
	ResourceStorage resourceStorage;

	// Use this for initialization
	void Start () {
		seeker = GetComponent<SeeksPosition>();
		resourceStorage = GetComponent<ResourceStorage>();
		unloadTimeLeft = unloadTime;
		loadTimeLeft = loadTime;
	}
	
	// Update is called once per frame
	void FixedUpdate () {
		if(stops==null || stops.Length==0)
			return;

		if(currStop > stops.Length-1)
			currStop = 0;

		Vector3 diff = this.transform.position - stops[currStop].transform.position;
		diff.y = 0;

		if(diff.magnitude < arrivalDistance) {
			unloadTimeLeft -= Time.deltaTime;
			loadTimeLeft -= Time.deltaTime;
			seeker.target = this.transform.position;
			ResourceStorage currentRS = stops[currStop].GetComponent<ResourceStorage>();

			if(unloading && unloadTimeLeft <= 0) {
				if(resourceStorage != null) {
					// Drop off!
					Debug.Log ("Delivered the following to "+ currentRS.gameObject.name +": " + resourceStorage.ToString());
					resourceStorage.DumpCargo(currentRS);
				}
				loadTimeLeft = loadTime;
				unloading = false;
			}

			if(unloading==false && loadTimeLeft <= 0) {
				currStop++;
				if(currStop > stops.Length-1)
					currStop = 0;
				ResourceStorage nextRS = stops[currStop].GetComponent<ResourceStorage>();

				if(resourceStorage != null) {
					// Pick up!  Loop until we're full or there's nothing left to pick up.
					while(currentRS.contentTotal >= 1 && resourceStorage.contentTotal < resourceStorage.storageLimit) {
						string name = currentRS.GetHighestDifferentialGood(nextRS);

						if(name==null || name=="") 
							break; // Remember to break if we don't find a good with higher demand!!

						currentRS.ChangeResource(name, -1);
						resourceStorage.ChangeResource(name, 1);
					}
				}

				Debug.Log ("Picked up the following from "+ currentRS.gameObject.name +": " + resourceStorage.ToString());

				unloadTimeLeft=unloadTime;
				unloading = true;
			}

		}
		else {
			seeker.target = stops[currStop].transform.position;
		}

	}

	override public string ToString() {
		string s = "";
		foreach(ResourceStorage rs in stops) {
			if(s != "")
				s += ", ";
			
			s += rs.name;
		}
		
		if(s=="")
			return "NONE";
		
		return s;

	}
}
