﻿using UnityEngine;
using System.Collections.Generic;

public class SkyCity : MonoBehaviour {

	[System.NonSerialized]
	public List<AirportFlocker> flockers;
	public float initialFood = 0;

	float flockerConstructionTimer = 10f;
	public GameObject flockerPrefab;
	ResourceStorage rs;
	float houseLossCooldown=5f;

	int basePopulation = 1000;

	[System.NonSerialized]
	//public float taxRate = 0.05f;

	float taxX = 1.0f;
	float taxDecayCooldown = 30;	

	public float taxRate {
		get {
			return Mathf.Clamp( 1f + Mathf.Log(taxX)/10f, 1.01f, 1.50f);
		}
	}

	float luxCooldown = 1f;

	void Start() {
		rs = GetComponent<ResourceStorage>();

		rs.ChangeResource ("Food", initialFood);

		basePopulation = Random.Range(900,1100);


		//rs.ChangeResource ("Steel", initialFood);
		//rs.ChangeResource ("Planks", initialFood);
		//rs.ChangeResource ("Fruit Cakes", initialFood);

	}


	public void RegisterFlocker(AirportFlocker af) {
		if(flockers==null)
			flockers = new List<AirportFlocker>();

		flockers.Add(af);
	}

	public void UnregisterFlocker(AirportFlocker af) {
		if(flockers==null)
			flockers = new List<AirportFlocker>();
		
		flockers.Remove(af);
	}

	void Update() {
		if(flockers == null)
			return;

		float foodNeed = flockers.Count/90f * Time.deltaTime;

		if(rs.GetResource("Food") < foodNeed) {
			houseLossCooldown -= Time.deltaTime;
			if(houseLossCooldown <= 0) {
				AirportFlocker af = flockers[0];
				flockers.Remove(af);
				Destroy (af.gameObject);
				DialogBox.ShowScript("Lost a house!", true);
				houseLossCooldown = 5f;
			}
		}
		else {
			rs.ChangeResource("Food", -foodNeed);
		}

		if(rs.GetResource("Steel") >= 4.99f && rs.GetResource("Planks") >= 4.99f) {
			flockerConstructionTimer -= Time.deltaTime;

			if(flockerConstructionTimer <= 0) {
				flockerConstructionTimer = flockers.Count * 5f;
				Vector3 pos = Random.insideUnitSphere * 20f;
				pos.y = 0;
				pos += transform.position;

				GameObject go = (GameObject)Instantiate(flockerPrefab, pos, Quaternion.identity);
				AirportFlocker af = go.GetComponent<AirportFlocker>();
				af.airport = this;

				rs.ChangeResource("Steel", -5f);
				rs.ChangeResource("Planks", -5f);

				DialogBox.ShowScript("New Housing!", true);
			}
		}

		luxCooldown -= Time.deltaTime;

		if(luxCooldown <= 0) {
			luxCooldown =  1f;
			foreach(string l in ResourceStorage.LUX_GOODS) {
				if(rs.GetResource(l) >= 1f) {
					rs.ChangeResource(l, -1f);

					//taxRate *= 1.05f;
					taxX += 1f;

					/*if(l=="Fruit Cakes") {
						rs.ChangeResource("Food", 1f);
					}*/
					DialogBox.ShowScript("Tax Increase!", true);

				}
			}
		}

		taxDecayCooldown -= Time.deltaTime;
		if(taxDecayCooldown <= 0) {
			if(taxX >= 2)
				taxX -= 1;

			taxDecayCooldown = 30f;
		}

	}

	public int GetPopulation() {
		return Mathf.FloorToInt(basePopulation * Mathf.Pow(1.5f, flockers.Count));
	}


}
